@@ -26,6 +26,7 @@
 #include "goaprovider-priv.h"
 #include "goaoauth2provider.h"
 #include "goagoogleprovider.h"
+#include "goaobjectskeletonutils.h"
 
 /**
  * GoaGoogleProvider:
@@ -322,13 +323,11 @@ build_object (GoaProvider         *provider,
 {
   GoaAccount *account;
   GoaMail *mail;
-  GoaCalendar *calendar;
-  GoaContacts *contacts;
+  gchar *uri_caldav;
   GoaChat *chat;
-  GoaDocuments *documents;
-  GoaFiles *files;
   GoaPhotos *photos;
   GoaPrinters *printers;
+  gchar *uri_drive;
   gboolean ret;
   gboolean mail_enabled;
   gboolean calendar_enabled;
@@ -342,11 +341,7 @@ build_object (GoaProvider         *provider,
 
   account = NULL;
   mail = NULL;
-  calendar = NULL;
-  contacts = NULL;
   chat = NULL;
-  documents = NULL;
-  files = NULL;
   photos = NULL;
   printers = NULL;
   ret = FALSE;
@@ -395,52 +390,17 @@ build_object (GoaProvider         *provider,
     }
 
   /* Calendar */
-  calendar = goa_object_get_calendar (GOA_OBJECT (object));
   calendar_enabled = g_key_file_get_boolean (key_file, group, "CalendarEnabled", NULL);
-  if (calendar_enabled)
-    {
-      if (calendar == NULL)
-        {
-          gchar *uri_caldav;
-
-          uri_caldav = g_strconcat ("https://apidata.googleusercontent.com/caldav/v2/",
-                                    email_address,
-                                    "/user",
-                                    NULL);
-
-          calendar = goa_calendar_skeleton_new ();
-          g_object_set (G_OBJECT (calendar),
-                        "uri", uri_caldav,
-                        NULL);
-          goa_object_skeleton_set_calendar (object, calendar);
-          g_free (uri_caldav);
-        }
-    }
-  else
-    {
-      if (calendar != NULL)
-        goa_object_skeleton_set_calendar (object, NULL);
-    }
+  uri_caldav = g_strconcat ("https://apidata.googleusercontent.com/caldav/v2/", email_address, "/user", NULL);
+  goa_object_skeleton_attach_calendar (object, uri_caldav, calendar_enabled, FALSE);
+  g_free (uri_caldav);
 
   /* Contacts */
-  contacts = goa_object_get_contacts (GOA_OBJECT (object));
   contacts_enabled = g_key_file_get_boolean (key_file, group, "ContactsEnabled", NULL);
-  if (contacts_enabled)
-    {
-      if (contacts == NULL)
-        {
-          contacts = goa_contacts_skeleton_new ();
-          g_object_set (G_OBJECT (contacts),
-                        "uri", "https://www.googleapis.com/.well-known/carddav",
-                        NULL);
-          goa_object_skeleton_set_contacts (object, contacts);
-        }
-    }
-  else
-    {
-      if (contacts != NULL)
-        goa_object_skeleton_set_contacts (object, NULL);
-    }
+  goa_object_skeleton_attach_contacts (object,
+                                       "https://www.googleapis.com/.well-known/carddav",
+                                       contacts_enabled,
+                                       FALSE);
 
 #ifdef GOA_TELEPATHY_ENABLED
   /* Chat */
@@ -464,22 +424,8 @@ build_object (GoaProvider         *provider,
 #endif
 
   /* Documents */
-  documents = goa_object_get_documents (GOA_OBJECT (object));
   documents_enabled = g_key_file_get_boolean (key_file, group, "DocumentsEnabled", NULL);
-
-  if (documents_enabled)
-    {
-      if (documents == NULL)
-        {
-          documents = goa_documents_skeleton_new ();
-          goa_object_skeleton_set_documents (object, documents);
-        }
-    }
-  else
-    {
-      if (documents != NULL)
-        goa_object_skeleton_set_documents (object, NULL);
-    }
+  goa_object_skeleton_attach_documents (object, documents_enabled);
 
   /* Photos */
   photos = goa_object_get_photos (GOA_OBJECT (object));
@@ -500,28 +446,10 @@ build_object (GoaProvider         *provider,
     }
 
   /* Files */
-  files = goa_object_get_files (GOA_OBJECT (object));
   files_enabled = g_key_file_get_boolean (key_file, group, "FilesEnabled", NULL);
-  if (files_enabled)
-    {
-      if (files == NULL)
-        {
-          gchar *uri_drive;
-
-          uri_drive = g_strconcat ("google-drive://", email_address, "/", NULL);
-          files = goa_files_skeleton_new ();
-          g_object_set (G_OBJECT (files),
-                        "uri", uri_drive,
-                        NULL);
-          goa_object_skeleton_set_files (object, files);
-          g_free (uri_drive);
-        }
-    }
-  else
-    {
-      if (files != NULL)
-        goa_object_skeleton_set_files (object, NULL);
-    }
+  uri_drive = g_strconcat ("google-drive://", email_address, "/", NULL);
+  goa_object_skeleton_attach_files (object, uri_drive, files_enabled, FALSE);
+  g_free (uri_drive);
 
   /* Printers */
   printers = goa_object_get_printers (GOA_OBJECT (object));
@@ -590,12 +518,8 @@ build_object (GoaProvider         *provider,
 
  out:
   g_clear_object (&printers);
-  g_clear_object (&files);
   g_clear_object (&photos);
-  g_clear_object (&documents);
   g_clear_object (&chat);
-  g_clear_object (&contacts);
-  g_clear_object (&calendar);
   g_clear_object (&mail);
   g_clear_object (&account);
   return ret;
