@@ -1,6 +1,6 @@
 /* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
 /*
- * Copyright (C) 2011, 2013, 2014 Red Hat, Inc.
+ * Copyright (C) 2011, 2013, 2014, 2015, 2016 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -292,7 +292,6 @@ ensure_credentials_sync (GoaProvider         *provider,
                          GError             **error)
 {
   GVariant *credentials;
-  GoaAccount *account;
   GoaMailAuth *imap_auth;
   GoaMailAuth *smtp_auth;
   GoaMailClient *mail_client;
@@ -301,7 +300,6 @@ ensure_credentials_sync (GoaProvider         *provider,
   gboolean imap_accept_ssl_errors;
   gboolean ret;
   gboolean smtp_accept_ssl_errors;
-  const gchar *identity;
   gchar *domain;
   gchar *email_address;
   gchar *imap_password;
@@ -326,11 +324,7 @@ ensure_credentials_sync (GoaProvider         *provider,
 
   ret = FALSE;
 
-  credentials = goa_utils_lookup_credentials_sync (provider,
-                                                   object,
-                                                   cancellable,
-                                                   error);
-  if (credentials == NULL)
+  if (!goa_utils_get_credentials (provider, object, "imap-password", NULL, &imap_password, cancellable, error))
     {
       if (error != NULL)
         {
@@ -340,43 +334,24 @@ ensure_credentials_sync (GoaProvider         *provider,
       goto out;
     }
 
-  account = goa_object_peek_account (object);
-  identity = goa_account_get_identity (account);
-
   mail_client = goa_mail_client_new ();
 
   /* IMAP */
 
-  if (!g_variant_lookup (credentials, "imap-password", "s", &imap_password))
-    {
-      if (error != NULL)
-        {
-          *error = g_error_new (GOA_ERROR,
-                                GOA_ERROR_NOT_AUTHORIZED,
-                                /* Translators: the first parameter is a field name. The second is
-                                 * a GOA account identifier. */
-                                _("Did not find %s with identity ‘%s’ in credentials"),
-                                "imap-password",
-                                identity);
-        }
-      goto out;
-    }
-
   imap_accept_ssl_errors = goa_util_lookup_keyfile_boolean (object, "ImapAcceptSslErrors");
   imap_server = goa_util_lookup_keyfile_string (object, "ImapHost");
   imap_username = goa_util_lookup_keyfile_string (object, "ImapUserName");
   imap_tls_type = get_tls_type_from_object (object, "ImapUseSsl", "ImapUseTls");
 
   imap_auth = goa_imap_auth_login_new (NULL, NULL, imap_username, imap_password);
-  ret = goa_mail_client_check_sync (mail_client,
-                                    imap_server,
-                                    imap_tls_type,
-                                    imap_accept_ssl_errors,
-                                    (imap_tls_type == GOA_TLS_TYPE_SSL) ? 993 : 143,
-                                    imap_auth,
-                                    cancellable,
-                                    error);
-  if (!ret)
+  if (!goa_mail_client_check_sync (mail_client,
+                                   imap_server,
+                                   imap_tls_type,
+                                   imap_accept_ssl_errors,
+                                   (imap_tls_type == GOA_TLS_TYPE_SSL) ? 993 : 143,
+                                   imap_auth,
+                                   cancellable,
+                                   error))
     {
       if (error != NULL)
         {
@@ -400,22 +375,14 @@ ensure_credentials_sync (GoaProvider         *provider,
   /* SMTP */
 
   if (!goa_util_lookup_keyfile_boolean (object, "SmtpUseAuth"))
-    {
-      ret = TRUE;
-      goto smtp_done;
-    }
+    goto smtp_done;
 
-  if (!g_variant_lookup (credentials, "smtp-password", "s", &smtp_password))
+  if (!goa_utils_get_credentials (provider, object, "smtp-password", NULL, &smtp_password, cancellable, error))
     {
       if (error != NULL)
         {
-          *error = g_error_new (GOA_ERROR,
-                                GOA_ERROR_NOT_AUTHORIZED,
-                                /* Translators: the first parameter is a field name. The second is
-                                 * a GOA account identifier. */
-                                _("Did not find %s with identity ‘%s’ in credentials"),
-                                "smtp-password",
-                                identity);
+          (*error)->domain = GOA_ERROR;
+          (*error)->code = GOA_ERROR_NOT_AUTHORIZED;
         }
       goto out;
     }
@@ -428,15 +395,14 @@ ensure_credentials_sync (GoaProvider         *provider,
   email_address = goa_util_lookup_keyfile_string (object, "EmailAddress");
   goa_utils_parse_email_address (email_address, NULL, &domain);
   smtp_auth = goa_smtp_auth_new (NULL, NULL, domain, smtp_username, smtp_password);
-  ret = goa_mail_client_check_sync (mail_client,
-                                    smtp_server,
-                                    smtp_tls_type,
-                                    smtp_accept_ssl_errors,
-                                    (smtp_tls_type == GOA_TLS_TYPE_SSL) ? 465 : 587,
-                                    smtp_auth,
-                                    cancellable,
-                                    error);
-  if (!ret)
+  if (!goa_mail_client_check_sync (mail_client,
+                                   smtp_server,
+                                   smtp_tls_type,
+                                   smtp_accept_ssl_errors,
+                                   (smtp_tls_type == GOA_TLS_TYPE_SSL) ? 465 : 587,
+                                   smtp_auth,
+                                   cancellable,
+                                   error))
     {
       if (error != NULL)
         {
@@ -448,7 +414,7 @@ ensure_credentials_sync (GoaProvider         *provider,
                            */
                           _("Invalid %s with username ‘%s’ (%s, %d): "),
                           "smtp-password",
-                          imap_username,
+                          smtp_username,
                           g_quark_to_string ((*error)->domain),
                           (*error)->code);
           (*error)->domain = GOA_ERROR;
@@ -462,6 +428,8 @@ ensure_credentials_sync (GoaProvider         *provider,
   if (out_expires_in != NULL)
     *out_expires_in = 0;
 
+  ret = TRUE;
+
  out:
   g_clear_object (&imap_auth);
   g_clear_object (&smtp_auth);
@@ -474,8 +442,7 @@ ensure_credentials_sync (GoaProvider         *provider,
   g_free (smtp_password);
   g_free (smtp_server);
   g_free (smtp_username);
-  if (credentials != NULL)
-    g_variant_unref (credentials);
+  g_clear_pointer (&credentials, (GDestroyNotify) g_variant_unref);
   return ret;
 }
 
@@ -646,6 +613,25 @@ create_encryption_ui (GtkWidget  *grid,
   gtk_combo_box_set_active_id (GTK_COMBO_BOX (*out_combo_box), "starttls");
 }
 
+static void
+show_progress_ui (GtkContainer *container, gboolean progress)
+{
+  GList *children;
+  GList *l;
+
+  children = gtk_container_get_children (container);
+  for (l = children; l != NULL; l = l->next)
+    {
+      GtkWidget *widget = GTK_WIDGET (l->data);
+      gdouble opacity;
+
+      opacity = progress ? 1.0 : 0.0;
+      gtk_widget_set_opacity (widget, opacity);
+    }
+
+  g_list_free (children);
+}
+
 static void
 create_account_details_ui (GoaProvider    *provider,
                            GtkDialog      *dialog,
@@ -755,19 +741,18 @@ create_account_details_ui (GoaProvider    *provider,
   gtk_dialog_set_response_sensitive (data->dialog, GTK_RESPONSE_OK, FALSE);
 
   data->progress_grid = gtk_grid_new ();
-  gtk_widget_set_no_show_all (data->progress_grid, TRUE);
   gtk_orientable_set_orientation (GTK_ORIENTABLE (data->progress_grid), GTK_ORIENTATION_HORIZONTAL);
   gtk_grid_set_column_spacing (GTK_GRID (data->progress_grid), 3);
   gtk_container_add (GTK_CONTAINER (grid0), data->progress_grid);
 
   spinner = gtk_spinner_new ();
+  gtk_widget_set_opacity (spinner, 0.0);
   gtk_widget_set_size_request (spinner, 20, 20);
-  gtk_widget_show (spinner);
   gtk_spinner_start (GTK_SPINNER (spinner));
   gtk_container_add (GTK_CONTAINER (data->progress_grid), spinner);
 
   label = gtk_label_new (_("Connecting…"));
-  gtk_widget_show (label);
+  gtk_widget_set_opacity (label, 0.0);
   gtk_container_add (GTK_CONTAINER (data->progress_grid), label);
 
   if (new_account)
@@ -851,7 +836,7 @@ dialog_response_cb (GtkDialog *dialog, gint response_id, gpointer user_data)
 }
 
 static void
-imap_check_cb (GObject *source_object, GAsyncResult *res, gpointer user_data)
+mail_check_cb (GObject *source_object, GAsyncResult *res, gpointer user_data)
 {
   GoaMailClient *client = GOA_MAIL_CLIENT (source_object);
   AddAccountData *data = user_data;
@@ -859,7 +844,7 @@ imap_check_cb (GObject *source_object, GAsyncResult *res, gpointer user_data)
   goa_mail_client_check_finish (client, res, &data->error);
   g_main_loop_quit (data->loop);
   gtk_widget_set_sensitive (data->forward_button, TRUE);
-  gtk_widget_hide (data->progress_grid);
+  show_progress_ui (GTK_CONTAINER (data->progress_grid), FALSE);
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -983,11 +968,11 @@ add_account (GoaProvider    *provider,
                          (imap_tls_type == GOA_TLS_TYPE_SSL) ? 993 : 143,
                          imap_auth,
                          data.cancellable,
-                         imap_check_cb,
+                         mail_check_cb,
                          &data);
 
   gtk_widget_set_sensitive (data.forward_button, FALSE);
-  gtk_widget_show (data.progress_grid);
+  show_progress_ui (GTK_CONTAINER (data.progress_grid), TRUE);
   g_main_loop_run (data.loop);
 
   if (g_cancellable_is_cancelled (data.cancellable))
@@ -1072,11 +1057,11 @@ add_account (GoaProvider    *provider,
                          (smtp_tls_type == GOA_TLS_TYPE_SSL) ? 465 : 587,
                          smtp_auth,
                          data.cancellable,
-                         imap_check_cb,
+                         mail_check_cb,
                          &data);
 
   gtk_widget_set_sensitive (data.forward_button, FALSE);
-  gtk_widget_show (data.progress_grid);
+  show_progress_ui (GTK_CONTAINER (data.progress_grid), TRUE);
   g_main_loop_run (data.loop);
 
   smtp_use_auth = goa_mail_auth_is_needed (smtp_auth);
@@ -1188,8 +1173,7 @@ add_account (GoaProvider    *provider,
 
   g_free (domain);
   g_free (data.account_object_path);
-  if (data.loop != NULL)
-    g_main_loop_unref (data.loop);
+  g_clear_pointer (&data.loop, (GDestroyNotify) g_main_loop_unref);
   g_clear_object (&data.cancellable);
   g_clear_object (&imap_auth);
   g_clear_object (&smtp_auth);
@@ -1329,11 +1313,11 @@ refresh_account (GoaProvider    *provider,
                          (imap_tls_type == GOA_TLS_TYPE_SSL) ? 993 : 143,
                          imap_auth,
                          data.cancellable,
-                         imap_check_cb,
+                         mail_check_cb,
                          &data);
 
   gtk_widget_set_sensitive (data.forward_button, FALSE);
-  gtk_widget_show (data.progress_grid);
+  show_progress_ui (GTK_CONTAINER (data.progress_grid), TRUE);
   g_main_loop_run (data.loop);
 
   if (g_cancellable_is_cancelled (data.cancellable))
@@ -1403,11 +1387,11 @@ refresh_account (GoaProvider    *provider,
                          (smtp_tls_type == GOA_TLS_TYPE_SSL) ? 465 : 587,
                          smtp_auth,
                          data.cancellable,
-                         imap_check_cb,
+                         mail_check_cb,
                          &data);
 
   gtk_widget_set_sensitive (data.forward_button, FALSE);
-  gtk_widget_show (data.progress_grid);
+  show_progress_ui (GTK_CONTAINER (data.progress_grid), TRUE);
   g_main_loop_run (data.loop);
 
   if (g_cancellable_is_cancelled (data.cancellable))
@@ -1473,8 +1457,7 @@ refresh_account (GoaProvider    *provider,
   g_free (imap_username);
   g_free (smtp_server);
   g_free (smtp_username);
-  if (data.loop != NULL)
-    g_main_loop_unref (data.loop);
+  g_clear_pointer (&data.loop, (GDestroyNotify) g_main_loop_unref);
   g_clear_object (&data.cancellable);
   g_clear_object (&imap_auth);
   g_clear_object (&smtp_auth);
@@ -1599,9 +1582,7 @@ on_handle_get_password (GoaPasswordBased      *interface,
   GoaAccount *account;
   GoaProvider *provider;
   GError *error;
-  GVariant *credentials;
   const gchar *account_id;
-  const gchar *identity;
   const gchar *method_name;
   const gchar *provider_type;
   gchar *password;
@@ -1609,12 +1590,9 @@ on_handle_get_password (GoaPasswordBased      *interface,
   /* TODO: maybe log what app is requesting access */
 
   password = NULL;
-  credentials = NULL;
 
   object = GOA_OBJECT (g_dbus_interface_get_object (G_DBUS_INTERFACE (interface)));
   account = goa_object_peek_account (object);
-  identity = goa_account_get_identity (account);
-
   account_id = goa_account_get_id (account);
   provider_type = goa_account_get_provider_type (account);
   method_name = g_dbus_method_invocation_get_method_name (invocation);
@@ -1623,35 +1601,16 @@ on_handle_get_password (GoaPasswordBased      *interface,
   provider = goa_provider_get_for_provider_type (provider_type);
 
   error = NULL;
-  credentials = goa_utils_lookup_credentials_sync (provider,
-                                                   object,
-                                                   NULL, /* GCancellable* */
-                                                   &error);
-  if (credentials == NULL)
+  if (!goa_utils_get_credentials (provider, object, id, NULL, &password, NULL, &error))
     {
       g_dbus_method_invocation_take_error (invocation, error);
       goto out;
     }
 
-  if (!g_variant_lookup (credentials, id, "s", &password))
-    {
-      g_dbus_method_invocation_return_error (invocation,
-                                             GOA_ERROR,
-                                             GOA_ERROR_FAILED, /* TODO: more specific */
-                                             /* Translators: the first parameter is a field name.
-                                              * The second is a GOA account identifier. */
-                                             "Did not find %s with identity ‘%s’ in credentials",
-                                             id,
-                                             identity);
-      goto out;
-    }
-
   goa_password_based_complete_get_password (interface, invocation, password);
 
  out:
   g_free (password);
-  if (credentials != NULL)
-    g_variant_unref (credentials);
   g_object_unref (provider);
   return TRUE; /* invocation was handled */
 }
