@@ -243,8 +243,6 @@ sign_in_identity (GoaKerberosProvider  *self,
                                        sign_in_thread,
                                        G_PRIORITY_DEFAULT,
                                        cancellable);
-
-  g_object_unref (operation_result);
 }
 
 static void
@@ -269,7 +267,7 @@ get_ticket_sync (GoaKerberosProvider *self,
                  GCancellable        *cancellable,
                  GError             **error)
 {
-  GVariant            *credentials = NULL;
+  GVariant            *credentials;
   GError              *lookup_error;
   GError              *sign_in_error;
   GoaAccount          *account;
@@ -283,7 +281,7 @@ get_ticket_sync (GoaKerberosProvider *self,
 
   ret = FALSE;
 
-  account = goa_object_get_account (object);
+  account = goa_object_peek_account (object);
   identifier = goa_account_get_identity (account);
 
   ticketing = goa_object_get_ticketing (object);
@@ -293,7 +291,7 @@ get_ticket_sync (GoaKerberosProvider *self,
                    GOA_ERROR,
                    GOA_ERROR_NOT_SUPPORTED,
                    _("Ticketing is disabled for account"));
-      goto out;
+      return FALSE;
     }
 
   details = goa_ticketing_get_details (ticketing);
@@ -352,7 +350,6 @@ get_ticket_sync (GoaKerberosProvider *self,
 
   ret = TRUE;
 out:
-  g_clear_object (&account);
   g_clear_object (&ticketing);
   g_free (object_path);
   g_clear_pointer (&credentials, (GDestroyNotify) g_variant_unref);
@@ -366,7 +363,7 @@ notify_is_temporary_cb (GObject *object, GParamSpec *pspec, gpointer user_data)
   gboolean is_temporary;
 
   account = GOA_ACCOUNT (object);
-  g_object_get (account, "is-temporary", &is_temporary, NULL);
+  is_temporary = goa_account_get_is_temporary (account);
 
   /* Toggle IsTemporary */
   goa_utils_keyfile_set_boolean (account, "IsTemporary", is_temporary);
@@ -843,7 +840,7 @@ on_initial_sign_in_done (GoaKerberosProvider *self,
   else if (remember_password)
     {
       GVariantBuilder  builder;
-      char            *object_path;
+      const char      *object_path;
 
       object_path = g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (result));
 
@@ -871,8 +868,6 @@ on_initial_sign_in_done (GoaKerberosProvider *self,
                                                        NULL,
                                                        NULL);
         }
-
-      g_free (object_path);
     }
 
   g_simple_async_result_complete_in_idle (operation_result);
@@ -1133,13 +1128,11 @@ add_account (GoaProvider    *provider,
   const char  *username;
   const char *provider_type;
   gchar      *principal;
-  gchar      *principal_for_display;
   gint        response;
   GVariant   *options;
 
   object = NULL;
   principal = NULL;
-  principal_for_display = NULL;
 
   memset (&request, 0, sizeof (SignInRequest));
   request.cancellable = g_cancellable_new ();
@@ -1194,9 +1187,6 @@ start_over:
   g_free (principal);
   principal = g_strdup_printf ("%s@%s", username, realm);
 
-  g_free (principal_for_display);
-  principal_for_display = g_strdup_printf ("%s@%s", username, realm);
-
   /* See if there's already an account of this type with the
    * given identity
    */
@@ -1204,7 +1194,7 @@ start_over:
 
   if (!goa_utils_check_duplicate (client,
                                   principal,
-                                  principal_for_display,
+                                  principal,
                                   provider_type,
                                   (GoaPeekInterfaceFunc) goa_object_peek_account,
                                   &request.error))
@@ -1224,7 +1214,7 @@ start_over:
   goa_manager_call_add_account (goa_client_get_manager (client),
                                 goa_provider_get_provider_type (provider),
                                 principal,
-                                principal_for_display,
+                                principal,
                                 g_variant_builder_end (&credentials),
                                 g_variant_builder_end (&details),
                                 NULL, /* GCancellable* */
@@ -1318,7 +1308,6 @@ start_over:
 
   g_free (request.account_object_path);
   g_free (principal);
-  g_free (principal_for_display);
   g_clear_pointer (&request.loop, (GDestroyNotify) g_main_loop_unref);
   return object;
 }
@@ -1366,14 +1355,14 @@ ensure_credentials_sync (GoaProvider    *provider,
                          GError        **error)
 {
   GoaIdentityServiceIdentity *identity = NULL;
-  GoaAccount                 *account = NULL;
+  GoaAccount                 *account;
   const char                 *identifier;
   gint64                      timestamp;
   GDateTime                  *now, *expiration_time;
   GTimeSpan                   time_span;
   gboolean                    credentials_ensured = FALSE;
 
-  account = goa_object_get_account (object);
+  account = goa_object_peek_account (object);
   identifier = goa_account_get_identity (account);
 
   ensure_identity_manager ();
@@ -1435,16 +1424,13 @@ ensure_credentials_sync (GoaProvider    *provider,
   if (time_span < 0 || time_span > G_MAXINT)
     time_span = 0;
 
-  if (out_expires_in != NULL)
-    *out_expires_in = (int) time_span;
-
+  *out_expires_in = (int) time_span;
   credentials_ensured = TRUE;
 
   g_date_time_unref (now);
   g_date_time_unref (expiration_time);
 
 out:
-  g_clear_object (&account);
   g_clear_object (&identity);
   g_mutex_unlock (&identity_manager_mutex);
   return credentials_ensured;
