@@ -254,17 +254,11 @@ ensure_credentials_sync (GoaProvider         *provider,
                          GCancellable        *cancellable,
                          GError             **error)
 {
-  GVariant *credentials;
-  GoaAccount *account;
-  const gchar *username;
+  gchar *username = NULL;
   gchar *password = NULL;
   gboolean ret = FALSE;
 
-  credentials = goa_utils_lookup_credentials_sync (provider,
-                                                   object,
-                                                   cancellable,
-                                                   error);
-  if (credentials == NULL)
+  if (!goa_utils_get_credentials (provider, object, "password", &username, &password, cancellable, error))
     {
       if (error != NULL)
         {
@@ -274,23 +268,7 @@ ensure_credentials_sync (GoaProvider         *provider,
       goto out;
     }
 
-  account = goa_object_peek_account (object);
-  username = goa_account_get_identity (account);
-
-  if (!g_variant_lookup (credentials, "password", "s", &password))
-    {
-      if (error != NULL)
-        {
-          *error = g_error_new (GOA_ERROR,
-                                GOA_ERROR_NOT_AUTHORIZED,
-                                _("Did not find password with identity ‘%s’ in credentials"),
-                                username);
-        }
-      goto out;
-    }
-
-  ret = lastfm_login_sync (provider, username, password, error);
-  if (!ret)
+  if (!lastfm_login_sync (provider, username, password, error))
     {
       if (error != NULL)
         {
@@ -313,9 +291,8 @@ ensure_credentials_sync (GoaProvider         *provider,
     *out_expires_in = 0;
 
  out:
+  g_free (username);
   g_free (password);
-  if (credentials != NULL)
-    g_variant_unref (credentials);
   return ret;
 }
 
@@ -760,8 +737,7 @@ add_account (GoaProvider    *provider,
 
   g_free (data.access_token);
   g_free (data.account_object_path);
-  if (data.loop != NULL)
-    g_main_loop_unref (data.loop);
+  g_clear_pointer (&data.loop, (GDestroyNotify) g_main_loop_unref);
   g_clear_object (&data.cancellable);
   return ret;
 }
@@ -891,8 +867,7 @@ refresh_account (GoaProvider    *provider,
     g_propagate_error (error, data.error);
 
   gtk_widget_destroy (dialog);
-  if (data.loop != NULL)
-    g_main_loop_unref (data.loop);
+  g_clear_pointer (&data.loop, (GDestroyNotify) g_main_loop_unref);
   g_free (data.access_token);
   g_clear_object (&data.cancellable);
   return ret;
