@@ -304,8 +304,6 @@ on_sign_in_handled (GoaIdentityService    *self,
                                                      invocation,
                                                      object_path);
     }
-
-  g_object_unref (invocation);
 }
 
 static void
@@ -444,7 +442,7 @@ on_identity_signed_out (GoaIdentityManager *manager,
   GError             *error;
   GoaIdentity        *identity;
   const char         *identifier;
-  GoaObject          *object = NULL;
+  GoaObject          *object;
 
   error = NULL;
   goa_identity_manager_sign_identity_out_finish (manager, result, &error);
@@ -466,8 +464,6 @@ on_identity_signed_out (GoaIdentityManager *manager,
     ensure_account_credentials (self, object);
 
   g_simple_async_result_complete_in_idle (operation_result);
-
-  g_clear_object (&object);
   g_object_unref (operation_result);
 }
 
@@ -751,20 +747,20 @@ on_identity_needs_renewal (GoaIdentityManager *identity_manager,
                            GoaIdentityService *self)
 {
   const char *principal;
-  GoaObject  *object = NULL;
+  GoaObject  *object;
 
   principal = goa_identity_get_identifier (identity);
 
+  g_debug ("GoaIdentityService: identity %s needs renewal", principal);
+
   object = find_object_with_principal (self, principal, TRUE);
 
-  if (object != NULL && should_ignore_object (self, object))
+  if (object != NULL)
     {
-      g_debug ("GoaIdentityService: ignoring identity %s that says it needs renewal", principal);
-      goto out;
+      should_ignore_object (self, object);
+      return;
     }
 
-  g_debug ("GoaIdentityService: identity %s needs renewal", principal);
-
   goa_identity_manager_renew_identity (GOA_IDENTITY_MANAGER
                                        (self->priv->identity_manager),
                                        identity,
@@ -772,9 +768,6 @@ on_identity_needs_renewal (GoaIdentityManager *identity_manager,
                                        (GAsyncReadyCallback)
                                        on_identity_renewed,
                                        self);
-
- out:
-  g_clear_object (&object);
 }
 
 static void
@@ -970,8 +963,6 @@ on_identity_added (GoaIdentityManager *identity_manager,
 
   if (object == NULL)
     add_temporary_account (self, identity);
-
-  g_clear_object (&object);
 }
 
 static void
@@ -989,7 +980,6 @@ on_identity_removed (GoaIdentityManager *identity_manager,
     ensure_account_credentials (self, object);
 
   unexport_identity (self, identity);
-  g_clear_object (&object);
 }
 
 static void
@@ -1007,8 +997,6 @@ on_identity_refreshed (GoaIdentityManager *identity_manager,
     add_temporary_account (self, identity);
   else
     ensure_account_credentials (self, object);
-
-  g_clear_object (&object);
 }
 
 typedef struct
@@ -1349,7 +1337,6 @@ on_identity_expiring (GoaIdentityManager *identity_manager,
     return;
 
   ensure_account_credentials (self, object);
-  g_clear_object (&object);
 }
 
 static void
@@ -1370,7 +1357,6 @@ on_identity_expired (GoaIdentityManager *identity_manager,
     return;
 
   ensure_account_credentials (self, object);
-  g_clear_object (&object);
 }
 
 static void
